# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc
from tempest.lib import exceptions
import testtools

from magnum_tempest_plugin.tests.api.v1.rbac import base


class MagnumServiceTest(base.RbacBaseTests, metaclass=abc.ABCMeta):

    """Tests for magnum-service with RBAC."""

    def __init__(self, *args, **kwargs):
        super(MagnumServiceTest, self).__init__(*args, **kwargs)

    @testtools.testcase.attr('negative')
    def test_magnum_service_list_needs_admin(self):
        self.assertRaises(exceptions.Forbidden,
                          self.service_member_client.magnum_service_list)

    @testtools.testcase.attr('positive')
    def test_magnum_service_list(self):
        # get json object
        resp, msvcs = self.service_admin_client.magnum_service_list()
        self.assertEqual(200, resp.status)
        mcond_svc = msvcs.mservices[0]
        self.assertEqual(mcond_svc['id'], 1)
        self.assertEqual('up', mcond_svc['state'])
        self.assertEqual('magnum-conductor', mcond_svc['binary'])
        self.assertGreater(mcond_svc['report_count'], 0)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from tempest import config  # noqa


magnum_scope_enforcement = cfg.BoolOpt('magnum',
                                       default=False,
                                       help="Does the Magnum service API "
                                            "policies enforce scope? "
                                            "This configuration value should "
                                            "be same as magnum.conf: "
                                            "[oslo_policy].enforce_scope "
                                            "option.")

service_option = cfg.BoolOpt(
    "magnum", default=True,
    help="Whether or not magnum is expected to be available")

magnum_group = cfg.OptGroup(name="magnum", title="Magnum Options")

MagnumGroup = [
    cfg.StrOpt("catalog_type",
               default="container-infra",
               help="Catalog type of the coe service."),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the coe service."),
    cfg.StrOpt("docker_storage_driver",
               choices=['devicemapper', 'overlay'],
               help="Docker storage driver."),
    cfg.StrOpt("image_id",
               default="fedora-coreos-latest",
               help="Image id to be used for ClusterTemplate."),
    cfg.StrOpt("nic_id",
               default="public",
               help="NIC id."),
    cfg.StrOpt("keypair_name",
               default="magnum-test",
               help="Keypair name to use to log into nova instances, the "
                    "keypair is created automatically if not exist."),
    cfg.StrOpt("flavor_id",
               default="ds2G",
               help="Flavor id to use for ClusterTemplate."),
    cfg.StrOpt("magnum_url",
               help="Bypass URL for Magnum to skip service catalog lookup"),
    cfg.StrOpt("master_flavor_id",
               default="ds2G",
               help="Master flavor id to use for ClusterTemplate."),
    cfg.StrOpt("dns_nameserver",
               default="8.8.8.8",
               help="DNS nameserver to use for ClusterTemplate."),
    cfg.BoolOpt("copy_logs",
                default=True,
                help="Specify whether to copy nova server logs on failure."),
    cfg.BoolOpt("copy_logs_success",
                default=True,
                help="Specify whether to copy nova server logs on success."),
    cfg.StrOpt("coe",
               default="kubernetes",
               help="Container Orchestration Engine"),
    cfg.StrOpt("network_driver",
               help="Network Driver"),
    cfg.StrOpt("cluster_template_id",
               help="UUID of cluster template used for the test."),
    cfg.IntOpt("cluster_creation_timeout",
               default=30,
               help="Timeout(in minutes) to wait for the cluster creation "
                    "finished."),
    cfg.StrOpt("labels",
               default='{}',
               help="A dict of labels to be defined in cluster template"),
]



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc
from urllib import parse

from tempest.lib.common import rest_client

from magnum_tempest_plugin.common import config


class MagnumClient(rest_client.RestClient, metaclass=abc.ABCMeta):
    """Abstract class responsible for setting up auth provider"""

    def __init__(self, auth_provider, **kwargs):
        service = kwargs.pop(
            "service"
        ) if "service" in kwargs else 'container-infra'
        region = kwargs.pop(
            "region"
        ) if "region" in kwargs else config.Config.region
        disable_validation = kwargs.pop(
            "disable_ssl_certificate_validation"
        ) if "disable_ssl_certificate_validation" in kwargs else True
        super(MagnumClient, self).__init__(
            auth_provider=auth_provider,
            service=service,
            region=region,
            disable_ssl_certificate_validation=disable_validation,
            **kwargs
        )

    @classmethod
    def deserialize(cls, resp, body, model_type):
        return resp, model_type.from_json(body)

    @property
    def tenant_id(self):
        return self.client.tenant_id

    @classmethod
    def add_filters(cls, url, filters):
        """add_filters adds dict values (filters) to url as query parameters

        :param url: base URL for the request
        :param filters: dict with var:val pairs to add as parameters to URL
        :returns: url string
        """
        return url + "?" + parse(filters)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import abc

from magnum_tempest_plugin.common.templates import cluster
from magnum_tempest_plugin.tests.api.v1.rbac import base

IDEMPOTENT_IDS = {
    "test_delete_cluster_for_nonexisting_cluster": (
        "09589881-e3c1-4507-9ded-55994e46963f"),
    "test_update_cluster_for_nonexisting_cluster": (
        "145d02ee-a804-476a-948f-727099c3525d"),
    "test_create_cluster_with_nonexisting_flavor": (
        "a08f129e-b0cb-411b-9cc0-e6b97db3d6d7"),
    "test_create_cluster_with_zero_masters": (
        "9eee202d-77a0-43e0-b5e3-657ab5d46835"),
    "test_create_cluster_with_zero_nodes": (
        "bdfb3894-d970-4e35-95b3-fec3e8a717ae"),
    "test_create_cluster_for_nonexisting_cluster_template": (
        "f3be4b27-62f1-46d4-8c07-8b7315a2b4c3"),
    "test_create_list_sign_delete_clusters": (
        "b03cb130-d2ca-4721-8298-b491020d72db"),
}


class ClusterTest(base.RbacBaseTests,
                  cluster.ClusterTestTemplate,
                  metaclass=abc.ABCMeta):

    """Tests for cluster CRUD with RBAC."""

    @classmethod
    def tearDownClass(cls):
        if cls.delete_template:
            cls._delete_cluster_template(cls.cluster_template.uuid)
        super(ClusterTest, cls).tearDownClass()

    def setUp(self):
        super(ClusterTest, self).setUp()
        self._setup_cluster_template()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import fixtures

import kubernetes
from kubernetes.client.rest import ApiException
from oslo_log import log as logging
from oslo_serialization import base64
from oslo_utils import uuidutils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions
import testtools
from yaml import safe_load

from magnum_tempest_plugin.common import config
from magnum_tempest_plugin.common import datagen
from magnum_tempest_plugin.common import utils
from magnum_tempest_plugin.tests.api import base


HEADERS = {'OpenStack-API-Version': 'container-infra latest',
           'Accept': 'application/json',
           'Content-Type': 'application/json'}

# Please overwrite this ID map to specific IDs for tests
IDEMPOTENT_IDS = {
    "test_delete_cluster_for_nonexisting_cluster": (
        "208fabdb-f7b8-4bdd-8762-fdf88be0fbca"),
    "test_update_cluster_for_nonexisting_cluster": (
        "71c75c03-090c-4f33-9aad-f2919a3b63d2"),
    "test_create_cluster_with_nonexisting_flavor": (
        "5e4e68d7-6b0a-4548-a462-2f5c83004509"),
    "test_create_cluster_with_zero_masters": (
        "395288a7-dc5d-474b-ba2b-09158553d549"),
    "test_create_cluster_with_zero_nodes": (
        "6201ac61-fd28-43a9-824a-2d452b2fe5c5"),
    "test_create_cluster_for_nonexisting_cluster_template": (
        "1c81a8b2-de86-4494-a8a7-b194f4c1a1f9"),
    "test_create_list_sign_delete_clusters": (
        "82bdca24-d732-41d1-bfc0-983ec8229ca0"),
}


class ClusterTestTemplate(base.BaseTempestTest):

    """Tests template for cluster CRUD."""

    LOG = logging.getLogger(__name__)
    delete_template = False

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.clusters = []

    @classmethod
    def _setup_cluster_template(cls):
        if config.Config.cluster_template_id:
            _, cls.cluster_template = \
                cls.ct_reader_client.get_cluster_template(
                    config.Config.cluster_template_id)
        else:
            model = datagen.valid_cluster_template()
            _, cls.cluster_template = cls._create_cluster_template(model)
            cls.delete_template = True

    def setUp(self):
        super().setUp()

        # NOTE (dimtruck) by default tempest sets timeout to 20 mins.
        # We need more time.
        test_timeout = 3600
        self.useFixture(fixtures.Timeout(test_timeout, gentle=True))

    def tearDown(self):
        try:
            cluster_list = self.clusters[:]
            for cluster_id in cluster_list:
                self._delete_cluster(cluster_id)
                self.clusters.remove(cluster_id)
        finally:
            super().tearDown()

    @classmethod
    def _create_cluster_template(cls, cm_model):
        cls.LOG.debug('We will create a clustertemplate for %s', cm_model)
        resp, model = cls.ct_member_client.post_cluster_template(
            cm_model)
        return resp, model

    @classmethod
    def _delete_cluster_template(cls, cm_id):
        cls.LOG.debug('We will delete a clustertemplate for %s', cm_id)
        resp, model = cls.ct_member_client.delete_cluster_template(
            cm_id)
        return resp, model

    def _create_cluster(self, cluster_model):
        self.LOG.debug('We will create cluster for %s', cluster_model)
        resp, model = self.cluster_member_client.post_cluster(cluster_model)
        self.LOG.debug('Response: %s', resp)
        self.assertEqual(202, resp.status)
        self.assertIsNotNone(model.uuid)
        self.assertTrue(uuidutils.is_uuid_like(model.uuid))
        self.clusters.append(model.uuid)
        self.cluster_uuid = model.uuid

        timeout = (config.Config.cluster_creation_timeout + 10) * 60
        try:
            self.cluster_admin_client.wait_for_created_cluster(
                model.uuid, delete_on_error=False, timeout=timeout)
        except Exception as e:
            self.LOG.debug("Cluster create exception: %s\n" % e)
            self.copy_logs_handler(
                lambda: list(
                    [self._get_cluster_by_id(model.uuid)[1].master_addresses,
                     self._get_cluster_by_id(model.uuid)[1].node_addresses]),
                self.cluster_template.coe,
                self.keypair)
            raise

        if config.Config.copy_logs_success:
            self.LOG.debug('Copying logs on success')
            self.copy_logs_handler(
                lambda: list(
                    [self._get_cluster_by_id(model.uuid)[1].master_addresses,
                     self._get_cluster_by_id(model.uuid)[1].node_addresses]),
                self.cluster_template.coe,
                self.keypair)

        return resp, model

    def _delete_cluster(self, cluster_id):
        self.LOG.debug('We will delete a cluster for %s', cluster_id)
        resp, model = self.cluster_member_client.delete_cluster(cluster_id)
        self.assertEqual(204, resp.status)

        self.cluster_admin_client.wait_for_cluster_to_delete(cluster_id)

        self.assertRaises(exceptions.NotFound,
                          self.cert_reader_client.get_cert,
                          cluster_id, headers=HEADERS)
        return resp, model

    def _get_cluster_by_id(self, cluster_id):
        resp, model = self.cluster_reader_client.get_cluster(cluster_id)
        return resp, model

    # (dimtruck) Combining all these tests in one because
    # they time out on the gate (2 hours not enough)
    @testtools.testcase.attr('positive')
    @testtools.testcase.attr('slow')
    @decorators.idempotent_id(
        IDEMPOTENT_IDS['test_create_list_sign_delete_clusters'])
    def test_create_list_sign_delete_clusters(self):
        gen_model = datagen.valid_cluster_data(
            cluster_template_id=self.cluster_template.uuid, node_count=1)

        # test cluster create
        _, cluster_model = self._create_cluster(gen_model)
        self.assertNotIn('status', cluster_model)

        _, cluster_model = self._get_cluster_by_id(cluster_model.uuid)

        # test cluster list
        resp, cluster_list_model = self.cluster_reader_client.list_clusters()
        self.assertEqual(200, resp.status)
        self.assertGreater(len(cluster_list_model.clusters), 0)
        self.assertIn(
            cluster_model.uuid, list([x['uuid']
                                      for x in cluster_list_model.clusters]))

        # test ca show
        resp, ca = self.cert_reader_client.get_cert(
            cluster_model.uuid, headers=HEADERS)
        self.LOG.debug("cert resp: %s", resp)
        self.assertEqual(200, resp.status)
        self.assertEqual(ca.cluster_uuid, cluster_model.uuid)
        self.assertIsNotNone(ca.pem)
        self.assertIn('-----BEGIN CERTIFICATE-----', ca.pem)
        self.assertIn('-----END CERTIFICATE-----', ca.pem)

        # test ca sign
        csr_sample = utils.generate_csr_and_key()
        cert_data_model = datagen.cert_data(cluster_model.uuid,
                                            csr_data=csr_sample['csr'])
        resp, cert_model = self.cert_member_client.post_cert(
            cert_data_model, headers=HEADERS)
        self.LOG.debug("cert resp: %s", resp)
        self.assertEqual(201, resp.status)
        self.assertEqual(cert_model.cluster_uuid, cluster_model.uuid)
        self.assertIsNotNone(cert_model.pem)
        self.assertIn('-----BEGIN CERTIFICATE-----', cert_model.pem)
        self.assertIn('-----END CERTIFICATE-----', cert_model.pem)

        # test Kubernetes API
        kube_cfg = """
---
apiVersion: v1
clusters:
  - cluster:
      certificate-authority-data: {ca}
      server: {api_address}
    name: {name}
contexts:
  - context:
      cluster: {name}
      user: admin
    name: default
current-context: default
kind: Config
preferences: {{}}
users:
  - name: admin
    user:
      client-certificate-data: {cert}
      client-key-data: {key}
""".format(name=cluster_model.name,
           api_address=cluster_model.api_address,
           key=base64.encode_as_text(csr_sample['private_key']),
           cert=base64.encode_as_text(cert_model.pem),
           ca=base64.encode_as_text(ca.pem))

        kube_cfg = safe_load(kube_cfg)
        kube_config = kubernetes.config.load_kube_config_from_dict(kube_cfg)

        # Get nodes and pods list using kubernetes python client
        with kubernetes.client.ApiClient(kube_config) as api_client:
            v1 = kubernetes.client.CoreV1Api(api_client)
            try:
                list_nodes = v1.list_node(pretty="true")
                with open("/tmp/magnum-nodes/api-list-nodes", "w") as outfile:
                    outfile.write(str(list_nodes))
                list_pods = v1.list_pod_for_all_namespaces(pretty="true")
                with open("/tmp/magnum-nodes/api-list-pods", "w") as outfile:
                    outfile.write(str(list_pods))
            except ApiException as e:
                print("Exception when calling CoreV1Api: %s\n" % e)
                raise

        # Get nodes and pods using kubectl
        with open("/tmp/magnum-nodes/kube.conf", "w") as outfile:
            outfile.write(str(kube_cfg))
        try:
            self.copy_pod_logs()
        except Exception as e:
            self.LOG.debug("kubectl exception: %s\n" % e)
            raise

        # test cluster delete
        self._delete_cluster(cluster_model.uuid)
        self.clusters.remove(cluster_model.uuid)

    @testtools.testcase.attr('negative')
    @decorators.idempotent_id(
        IDEMPOTENT_IDS['test_create_cluster_for_nonexisting_cluster_template'])
    def test_create_cluster_for_nonexisting_cluster_template(self):
        cm_id = 'this-does-not-exist'
        gen_model = datagen.valid_cluster_data(cluster_template_id=cm_id)
        self.assertRaises(
            exceptions.BadRequest,
            self.cluster_member_client.post_cluster, gen_model)

    @testtools.testcase.attr('positive')
    @testtools.testcase.attr('slow')
    @decorators.idempotent_id(
        IDEMPOTENT_IDS['test_create_cluster_with_zero_nodes'])
    def test_create_cluster_with_zero_nodes(self):
        gen_model = datagen.valid_cluster_data(
            cluster_template_id=self.cluster_template.uuid, node_count=0)

        # test cluster create
        _, cluster_model = self._create_cluster(gen_model)
        self.assertNotIn('status', cluster_model)

        # test cluster delete
        self._delete_cluster(cluster_model.uuid)
        self.clusters.remove(cluster_model.uuid)

    @testtools.testcase.attr('negative')
    @decorators.idempotent_id(
        IDEMPOTENT_IDS['test_create_cluster_with_zero_masters'])
    def test_create_cluster_with_zero_masters(self):
        uuid = self.cluster_template.uuid
        gen_model = datagen.valid_cluster_data(cluster_template_id=uuid,
                                               master_count=0)
        self.assertRaises(
            exceptions.BadRequest,
            self.cluster_member_client.post_cluster, gen_model)

    @testtools.testcase.attr('negative')
    @decorators.idempotent_id(
        IDEMPOTENT_IDS['test_create_cluster_with_nonexisting_flavor'])
    def test_create_cluster_with_nonexisting_flavor(self):
        gen_model = \
            datagen.cluster_template_data_with_valid_keypair_image_flavor()
        resp, cluster_template = self._create_cluster_template(gen_model)
        self.assertEqual(201, resp.status)
        self.assertIsNotNone(cluster_template.uuid)

        uuid = cluster_template.uuid
        gen_model = datagen.valid_cluster_data(cluster_template_id=uuid)
        gen_model.flavor_id = 'aaa'
        self.assertRaises(exceptions.BadRequest,
                          self.cluster_member_client.post_cluster, gen_model)

        resp, _ = self._delete_cluster_template(cluster_template.uuid)
        self.assertEqual(204, resp.status)

    @testtools.testcase.attr('negative')
    @decorators.idempotent_id(
        IDEMPOTENT_IDS['test_update_cluster_for_nonexisting_cluster'])
    def test_update_cluster_for_nonexisting_cluster(self):
        patch_model = datagen.cluster_name_patch_data()

        self.assertRaises(
            exceptions.NotFound,
            self.cluster_member_client.patch_cluster, 'fooo', patch_model)

    @testtools.testcase.attr('negative')
    @decorators.idempotent_id(
        IDEMPOTENT_IDS['test_delete_cluster_for_nonexisting_cluster'])
    def test_delete_cluster_for_nonexisting_cluster(self):
        self.assertRaises(
            exceptions.NotFound,
            self.cluster_member_client.delete_cluster, data_utils.rand_uuid())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from magnum_tempest_plugin.tests.api.v1.clients.cert_client \
    import CertClient
from magnum_tempest_plugin.tests.api.v1.clients.cluster_client \
    import ClusterClient
from magnum_tempest_plugin.tests.api.v1.clients.cluster_template_client \
    import ClusterTemplateClient
from magnum_tempest_plugin.tests.api.v1.clients.magnum_service_client \
    import MagnumServiceClient


__all__ = [
    'CertClient',
    'ClusterClient',
    'ClusterTemplateClient',
    'MagnumServiceClient'
]



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from magnum_tempest_plugin.common.templates import cluster

IDEMPOTENT_IDS = {
    "test_delete_cluster_for_nonexisting_cluster": (
        "376c45ea-a857-11e9-9382-00224d6b7bc1"),
    "test_update_cluster_for_nonexisting_cluster": (
        "33bd0416-a857-11e9-9382-00224d6b7bc1"),
    "test_create_cluster_with_nonexisting_flavor": (
        "2cf16528-a857-11e9-9382-00224d6b7bc1"),
    "test_create_cluster_with_zero_masters": (
        "29c6c5f0-a857-11e9-9382-00224d6b7bc1"),
    "test_create_cluster_with_zero_nodes": (
        "262eb132-a857-11e9-9382-00224d6b7bc1"),
    "test_create_cluster_for_nonexisting_cluster_template": (
        "11c293da-a857-11e9-9382-00224d6b7bc1"),
    "test_create_list_sign_delete_clusters": (
        "44158a8c-a856-11e9-9382-00224d6b7bc1"),
}


class ClusterTest(cluster.ClusterTestTemplate):

    """Tests for cluster CRUD."""

    @classmethod
    def setUpClass(cls):
        super(ClusterTest, cls).setUpClass()

        try:
            (cls.creds, cls.keypair) = cls.get_credentials_with_keypair(
                type_of_creds='default'
            )
            (cls.cluster_template_client,
             cls.keypairs_client) = cls.get_clients_with_existing_creds(
                creds=cls.creds,
                type_of_creds='default',
                request_type='cluster_template'
            )
            cls.ct_reader_client = cls.cluster_template_client
            cls.ct_member_client = cls.cluster_template_client
            cls.ct_admin_client = cls.cluster_template_client
            (cls.cluster_client, _) = cls.get_clients_with_existing_creds(
                creds=cls.creds,
                type_of_creds='default',
                request_type='cluster'
            )
            cls.cluster_reader_client = cls.cluster_client
            cls.cluster_member_client = cls.cluster_client
            cls.cluster_admin_client = cls.cluster_client
            (cls.cert_client, _) = cls.get_clients_with_existing_creds(
                creds=cls.creds,
                type_of_creds='default',
                request_type='cert'
            )
            cls.cert_reader_client = cls.cert_client
            cls.cert_member_client = cls.cert_client
            cls.cert_admin_client = cls.cert_client

            cls._setup_cluster_template()
        except Exception:
            raise



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from magnum_tempest_plugin.common.templates import cluster_template_admin


class ClusterTemplateAdminTest(
        cluster_template_admin.ClusterTemplateAdminTestTemplate
):

    """Tests for clustertemplate admin operations."""

    def __init__(self, *args, **kwargs):
        super(ClusterTemplateAdminTest, self).__init__(*args, **kwargs)
        self.cluster_template_client = None
        self.keypairs_client = None

    def setUp(self):
        try:
            super(ClusterTemplateAdminTest, self).setUp()
            (self.cluster_template_client,
             self.keypairs_client) = self.get_clients_with_new_creds(
                 type_of_creds='admin',
                 request_type='cluster_template')
            self.ct_reader_client = self.cluster_template_client
            self.ct_member_client = self.cluster_template_client
            self.ct_admin_client = self.cluster_template_client
        except Exception:
            self.tearDown()
            raise



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
import functools
import inspect
import time
import types

from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization
from cryptography import x509


def def_method(f, *args, **kwargs):
    @functools.wraps(f)
    def new_method(self):
        return f(self, *args, **kwargs)
    return new_method


def parameterized_class(cls):
    """A class decorator for running parameterized test cases.

    Mark your class with @parameterized_class.
    Mark your test cases with @parameterized.
    """
    test_functions = inspect.getmembers(cls, predicate=inspect.ismethod)
    for (name, f) in test_functions:
        if name.startswith('test_') and not hasattr(f, '_test_data'):
            continue

        # remove the original test function from the class
        delattr(cls, name)

        # add a new test function to the class for each entry in f._test_data
        for tag, args in f._test_data.items():
            new_name = "{0}_{1}".format(f.__name__, tag)
            if hasattr(cls, new_name):
                raise Exception(
                    "Parameterized test case '{0}.{1}' created from '{0}.{2}' "
                    "already exists".format(cls.__name__, new_name, name))

            # Using `def new_method(self): f(self, **args)` is not sufficient
            # (all new_methods use the same args value due to late binding).
            # Instead, use this factory function.
            new_method = def_method(f, **args)

            # To add a method to a class, available for all instances:
            #   MyClass.method = types.MethodType(f, None, MyClass)
            setattr(cls, new_name, types.MethodType(new_method, None, cls))
    return cls


def parameterized(data):
    """A function decorator for parameterized test cases.

    Example:

        @parameterized({
            'zero': dict(val=0),
            'one': dict(val=1),
        })
        def test_val(self, val):
            self.assertEqual(val, self.get_val())

    The above will generate two test cases:
        `test_val_zero` which runs with val=0
        `test_val_one` which runs with val=1

    :param data: A dictionary that looks like {tag: {arg1: val1, ...}}
    """
    def wrapped(f):
        f._test_data = data
        return f
    return wrapped


def wait_for_condition(condition, interval=1, timeout=40):
    start_time = time.time()
    end_time = time.time() + timeout
    while time.time() < end_time:
        result = condition()
        if result:
            return result
        time.sleep(interval)
    raise Exception(("Timed out after %s seconds.  Started on %s "
                    + "and ended on %s") % (timeout, start_time, end_time))


def memoized(func):
    """A decorator to cache function's return value"""
    cache = {}

    @functools.wraps(func)
    def wrapper(*args):
        if not isinstance(args, collections.Hashable):
            # args is not cacheable. just call the function.
            return func(*args)
        if args in cache:
            return cache[args]
        else:
            value = func(*args)
            cache[args] = value
            return value
    return wrapper


def generate_csr_and_key():
    """Return a dict with a new csr, public key and private key."""
    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048
    )

    public_key = private_key.public_key()

    csr = x509.CertificateSigningRequestBuilder().subject_name(x509.Name([
        x509.NameAttribute(x509.oid.NameOID.COMMON_NAME, u"admin"),
        x509.NameAttribute(x509.oid.NameOID.ORGANIZATION_NAME,
                           u"system:masters")
    ])).sign(private_key, hashes.SHA256())

    result = {
        'csr': csr.public_bytes(
            encoding=serialization.Encoding.PEM).decode("utf-8"),
        'private_key': private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.NoEncryption()).decode("utf-8"),
        'public_key': public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo).decode(
                "utf-8"),
    }

    return result



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os

from tempest import config
from tempest.test_discover import plugins

from magnum_tempest_plugin import config as magnum_config


class MagnumTempestPlugin(plugins.TempestPlugin):
    def load_tests(self):
        base_path = os.path.split(os.path.dirname(
            os.path.abspath(__file__)))[0]
        test_dir = "magnum_tempest_plugin/tests/api/v1"
        full_test_dir = os.path.join(base_path, test_dir)
        return full_test_dir, base_path

    def register_opts(self, conf):
        conf.register_opt(magnum_config.service_option,
                          group='service_available')
        conf.register_opt(magnum_config.magnum_scope_enforcement,
                          group='enforce_scope')
        conf.register_group(magnum_config.magnum_group)
        conf.register_opts(magnum_config.MagnumGroup, group='magnum')

    def get_opt_lists(self):
        return [
            (magnum_config.magnum_group.name, magnum_config.MagnumGroup),
            ('service_available', [magnum_config.service_option]),
            ('enforce_scope', [magnum_config.magnum_scope_enforcement])
        ]

    def get_service_clients(self):
        magnum_config = config.service_client_config('magnum')
        v1_params = {
            'name': 'magnum_v1',
            'service_version': 'magnum.v1',
            'module_path': 'magnum_tempest_plugin.tests.api.v1.clients',
            'client_names': [
                'CertClient', 'ClusterTemplateClient',
                'ClusterClient', 'MagnumServiceClient'
            ]
        }
        v1_params.update(magnum_config)
        return [v1_params]



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import clients
from tempest import config
from tempest.lib import auth
from tempest.lib.common.utils import data_utils

from magnum_tempest_plugin.common import base
from magnum_tempest_plugin.common import config as magnum_config

CONF = config.CONF


class RbacBaseTests(base.BaseMagnumTest):

    credentials = ['project_admin', 'project_member', 'project_reader']
    identity_version = 'v3'

    @classmethod
    def skip_checks(cls):
        super(RbacBaseTests, cls).skip_checks()
        if not CONF.enforce_scope.magnum:
            raise cls.skipException(
                "Tempest is not configured to enforce_scope for magnum, "
                "skipping RBAC tests. To enable these tests set "
                "`tempest.conf [enforce_scope] magnum=True`."
            )

    @classmethod
    def setUpClass(cls):
        magnum_config.Config.setUp()
        super(RbacBaseTests, cls).setUpClass()

    @classmethod
    def setup_clients(cls):
        super(RbacBaseTests, cls).setup_clients()
        # default to os_project_admin in base class
        cls.persona = getattr(cls, f"os_{cls.credentials[0]}")
        cls.magnum_clients = cls.persona.magnum_v1
        cls.project_admin = cls.os_project_admin.magnum_v1
        cls.project_reader = cls.os_project_reader.magnum_v1
        cls.project_member = cls.os_project_member.magnum_v1

        cls.ct_reader_client = cls.project_reader.ClusterTemplateClient()
        cls.ct_member_client = cls.project_member.ClusterTemplateClient()
        cls.ct_admin_client = cls.project_admin.ClusterTemplateClient()
        cls.cluster_reader_client = cls.project_reader.ClusterClient()
        cls.cluster_member_client = cls.project_member.ClusterClient()
        cls.cluster_admin_client = cls.project_admin.ClusterClient()
        cls.cert_reader_client = cls.project_reader.CertClient()
        cls.cert_member_client = cls.project_member.CertClient()
        cls.cert_admin_client = cls.project_admin.CertClient()
        cls.service_member_client = cls.project_member.MagnumServiceClient()
        cls.service_admin_client = cls.project_admin.MagnumServiceClient()

    @classmethod
    def setup_credentials(cls):
        super().setup_credentials()
        cls.os_primary = getattr(cls, f'os_{cls.credentials[0]}')

    def do_request(self, method, expected_status=200, client=None, **payload):
        if not client:
            client = self.client
        if isinstance(expected_status, type(Exception)):
            self.assertRaises(expected_status,
                              getattr(client, method),
                              **payload)
        else:
            response, mode = getattr(client, method)(**payload)
            self.assertEqual(response.status, expected_status)
            return response, mode

    def setup_user_client(self, project_id=None):
        """Set up project user with its own client.

        This is useful for testing protection of resources in separate
        projects.

        Returns a client object and the user's ID.
        """
        user_dict = {
            'name': data_utils.rand_name('user'),
            'password': data_utils.rand_password(),
        }
        user_id = self.os_system_admin.users_v3_client.create_user(
            **user_dict)['user']['id']
        self.addCleanup(self.os_system_admin.users_v3_client.delete_user,
                        user_id)

        if not project_id:
            project_id = self.os_system_admin.projects_client.create_project(
                data_utils.rand_name())['project']['id']
            self.addCleanup(
                self.os_system_admin.projects_client.delete_project,
                project_id)

        member_role_id = self.os_system_admin.roles_v3_client.list_roles(
            name='member')['roles'][0]['id']
        self.os_system_admin.roles_v3_client.create_user_role_on_project(
            project_id, user_id, member_role_id)
        creds = auth.KeystoneV3Credentials(
            user_id=user_id,
            password=user_dict['password'],
            project_id=project_id)
        auth_provider = clients.get_auth_provider(creds)
        creds = auth_provider.fill_credentials()
        client = clients.Manager(credentials=creds)
        return client



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import testtools

from magnum_tempest_plugin.common import datagen
from magnum_tempest_plugin.tests.api import base


class ClusterTemplateAdminTestTemplate(base.BaseTempestTest):

    """Template for Tests for clustertemplate admin operations."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.cluster_templates = []

    def tearDown(self):
        for cluster_template_id in self.cluster_templates:
            self._delete_cluster_template(cluster_template_id)
            self.cluster_templates.remove(cluster_template_id)
        super().tearDown()

    def _create_cluster_template(self, cmodel_model):
        resp, model = \
            self.ct_admin_client.post_cluster_template(cmodel_model)
        self.assertEqual(201, resp.status)
        self.cluster_templates.append(model.uuid)
        return resp, model

    def _delete_cluster_template(self, model_id):
        resp, model = \
            self.ct_admin_client.delete_cluster_template(model_id)
        self.assertEqual(204, resp.status)
        return resp, model

    @testtools.testcase.attr('positive')
    def test_create_get_public_cluster_template(self):
        gen_model = datagen.valid_cluster_template(is_public=True)
        resp, model = self._create_cluster_template(gen_model)

        resp, model = \
            self.ct_admin_client.get_cluster_template(model.uuid)
        self.assertEqual(200, resp.status)
        self.assertTrue(model.public)

    @testtools.testcase.attr('positive')
    def test_update_cluster_template_public_by_uuid(self):
        path = "/public"
        gen_model = \
            datagen.cluster_template_data_with_valid_keypair_image_flavor()
        resp, old_model = self._create_cluster_template(gen_model)

        patch_model = datagen.cluster_template_replace_patch_data(path,
                                                                  value=True)
        resp, new_model = self.ct_admin_client.patch_cluster_template(
            old_model.uuid, patch_model)
        self.assertEqual(200, resp.status)

        resp, model = self.ct_admin_client.get_cluster_template(
            new_model.uuid)
        self.assertEqual(200, resp.status)
        self.assertTrue(model.public)
